###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.8 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import OFS.DTMLMethod
import address_book_contact
import base_item
import string
import address_book_group_container

address_book_add_redirect = HTMLFile('address_book_add_redirect', globals())
manage_add_address_book_form = HTMLFile('address_book_add', globals())


def manage_add_address_book(self, id, title, REQUEST=None, RESPONSE=None):
    "Add an instance of the address_book class"
    self._setObject(id, address_book(id, title))

    if RESPONSE is not None:

		return RESPONSE.redirect(self.absolute_url() + '/' + str(id))


class dummy_group:

	def __init__(self):

		self.id = ''
		self.name = 'No specific group'


def name_match(object, letter):
	"""Filters out objects somewhat intelligently."""

	name = object.last_name

	if not string.strip(name):

		name = object.first_name

	if string.strip(name):

		return string.upper(name[0]) == letter


class address_book(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	product_name = 'AddressBook'

	meta_type = 'AddressBook'
	icon = 'misc_/AddressBook/Folder_icon.gif'

	letters = ['A','B','C','D','E','F','G','H','I','J','K','L','M',
			'N','O','P','Q','R','S','T','U','V','W','X','Y','Z']

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'groups', 'type':'lines'},
		)

	# FIXME
	#meta_types = (
	#{'name':'AddressBookContactSort', 'action':'manage_add_address_book_contact_sort_form'},
	#)

	meta_types = (
		{'name':'AddressBookContact', 'action':'manage_add_address_book_contact_form'},
		)

	manage_add_address_book_contact_form = address_book_contact.manage_add_address_book_contact_form
	manage_add_address_book_contact = address_book_contact.manage_add_address_book_contact
	address_book_contact_add_company_information = address_book_contact.address_book_contact_add_company_information

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add AddressBookContact',
		 ('manage_add_address_book_contact', 'manage_add_address_book_contact_form')),
		('Use AddressBook',
		 ('', 'address_book_edit', 'list_names', 'display',
			'sort_names', 'get_address_book_contacts',
			'get_unique_id', 'get_contacts_from_group',
			'get_contact_listing', 'get_all_groups',
			'render_group_widget', 'get_address_book_letters',
			)),
		)

	index_html = HTMLFile('address_book_index', globals())
	address_book_edit = base_edit = HTMLFile('address_book_edit', globals())

	list_names = HTMLFile('address_book_list_names', globals())
	display = HTMLFile('address_book_display', globals())


	def sort_names(self, name_starts_with):
		"""Returns a sorted list."""

		matched_contacts = []

		for contact in self.get_address_book_contacts():

			if name_match(contact, name_starts_with):

				matched_contacts.append(contact)

		return matched_contacts


	def get_address_book_contacts(self):
		"""Returns a list of objects."""

		return self.objectValues('AddressBookContact')


	def get_unique_id(self):
		"""Returns a unique identifier."""


		try:

			return self.getParentNode().get_unique_id()

		except AttributeError:

			self.counter = self.counter + 1

			return self.counter


	def get_contacts_from_group(self, group):
		"""Returns contacts for given group."""

		filter_function = eval("""lambda contact: '%s' in contact.groups""" % group)

		contacts = filter(filter_function, self.get_address_book_contacts())

		return contacts


	def get_contact_listing(self, REQUEST):
		"""Returns contacts from groups selected."""

		if REQUEST.has_key('n') and REQUEST['n']:

			return self.sort_names(REQUEST['n'])

		contacts = []

		if REQUEST.has_key('group') and REQUEST['group']:

			groups = REQUEST['group']

		else:

			return self.get_address_book_contacts()

		if not type(groups) == type([]):

			groups = [groups]

		for group in groups:

			contacts = contacts + self.get_contacts_from_group(group)


		return contacts


	def get_all_groups(self):
		"""Returns all known groups."""

		return self.groups.objectValues('AddressBookGroup')


	def render_group_widget(self, REQUEST):
		"""Returns an HTML widget for selecting groups."""

		if REQUEST.has_key('group'):

			request_group = REQUEST['group']

		else:

			request_group = ''			

		select_widget = """<select name="group">"""

		for group in [dummy_group()] + self.get_all_groups():

			if request_group == group.id:

				selected = " selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s">%s</option>""" % \
						(group.id, group.name)

		return select_widget + "</select>"


	def get_address_book_letters(self):
		"""Returns a list of address book letters."""

		return self.letters		


	def __init__(self, id, title=''):
		"Initialize the class instance"

		self.id = id
		self.title = title

		self.counter = 0

		address_book_group_container.manage_add_address_book_group_container(self, 'Contact groups')


		"""
		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_header',
				'standard_html_header',
				 open('%s/Products/%s/standard_html_header.dtml' % \
				 	(SOFTWARE_HOME, self.product_name)).read())
										  
		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_footer',
				'standard_html_footer',
				 open('%s/Products/%s/standard_html_footer.dtml' % \
					(SOFTWARE_HOME, self.product_name)).read())

		OFS.DTMLMethod.addDTMLMethod(self, 'standard_css',
				'standard_css',
				open('%s/Products/%s/standard_css.dtml' % \
					(SOFTWARE_HOME, self.product_name)).read())
		"""

		try:

			self.index_object()

		except AttributeError:

			# No ZCatalog instance available..

			pass


default__class_init__(address_book)
