###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.5 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import MessageDialog
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.SimpleItem
import base_item
import string

address_book_contact_add_company_information = HTMLFile('address_book_contact_add_company_information',
							globals())
manage_add_address_book_contact_form = HTMLFile('address_book_contact_add', globals())


def manage_add_address_book_contact(self, first_name='', middle_name='', last_name='', street='',
				city='', state='', zip='', country='', email='', website='',
				telephone_home='', telephone_mobile='', telephone_pager='',
				telephone_fax='', note='', assistant_name='', assistant_phone='', assistant_email='',
				company_name='', company_street='', company_city='', company_state='',
				company_zip='', company_country='', company_employee_job_title='',
				company_employee_department='', company_employee_pager='',
				company_employee_email='', company_employee_phone='',
				company_employee_fax='', REQUEST=None, RESPONSE=None):
    "Add an instance of the AddressBookContact class"

    id = str(self.get_unique_id())

    self._setObject(id,	address_book_contact(id, first_name, middle_name, last_name, street, city, state,
						zip, country, email, website, telephone_home,
						telephone_mobile, telephone_pager, telephone_fax,
						note, assistant_name, assistant_phone, assistant_email,
						company_name, company_street, company_city,
						company_state, company_zip, company_country,
						company_employee_job_title, company_employee_department,
						company_employee_pager, company_employee_email,
						company_employee_phone, company_employee_fax))

    if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url() + '/' + id)


class address_book_contact(
	OFS.SimpleItem.Item,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'AddressBookContact'
	icon = 'misc_/AddressBook/dtmldoc.gif'

	_properties = (
		{'id':'first_name', 'type':'string'},
		{'id':'middle_name', 'type':'string'},
		{'id':'last_name', 'type':'string'},
		{'id':'street', 'type':'string'},
		{'id':'city', 'type':'string'},
		{'id':'state', 'type':'string'},
		{'id':'zip', 'type':'string'},
		{'id':'country', 'type':'string'},
		{'id':'email', 'type':'string'},
		{'id':'website', 'type':'string'},
		{'id':'telephone_home', 'type':'string'},
		{'id':'telephone_mobile', 'type':'string'},
		{'id':'telephone_pager', 'type':'string'},
		{'id':'telephone_fax', 'type':'string'},
		{'id':'note', 'type':'text'},
		{'id':'assistant_name', 'type':'string'},
		{'id':'assistant_phone', 'type':'string'},
		{'id':'assistant_email', 'type':'string'},
		{'id':'company_name', 'type':'string'},
		{'id':'company_street', 'type':'string'},
		{'id':'company_city', 'type':'string'},
		{'id':'company_state', 'type':'string'},
		{'id':'company_zip', 'type':'string'},
		{'id':'company_country', 'type':'string'},
		{'id':'company_employee_job_title', 'type':'string'},
		{'id':'company_employee_department', 'type':'string'},
		{'id':'company_employee_pager', 'type':'string'},
		{'id':'company_employee_email', 'type':'string'},
		{'id':'company_employee_phone', 'type':'string'},
		{'id':'company_employee_fax', 'type':'string'},
		{'id':'groups', 'type':'tokens'}
		)

	manage_options = (
		{'label':'Edit', 'action':'manage_main'},
		{'label':'View', 'action':''},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View AddressBookContacts',
		('', 'render_group_widget', 'render_website')),
		('Manage AddressBookContacts',
		 ('edit', 'edit_company_information',
			'edit_note', 'edit_groups',
			'update_groups', 'copy')),
		)

	index_html = HTMLFile('address_book_contact_index', globals())
	edit = base_edit = HTMLFile('address_book_contact_edit', globals())
	edit_company_information = HTMLFile('address_book_contact_edit_company_information', globals())
	edit_note = HTMLFile('address_book_contact_edit_note', globals())
	edit_groups = HTMLFile('address_book_contact_edit_groups', globals())


	def render_group_widget(self):
		"""Returns an HTML widget for editing groups this contact is a part of."""

		select_widget = """<select name="groups" size="10" multiple><option value="">- - - - - - - - - - - - </option>"""
		select_widget = select_widget + """<!-- %s %s -->""" % (map(lambda x: (x.id, x.name), self.get_all_groups()), self.groups)

		for group in self.get_all_groups():

			if group.id in self.groups:

				selected = " selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s"%s>%s</option>""" % \
								(group.id, selected, group.name)

		return select_widget + "</select>"


	def update_groups(self, REQUEST, RESPONSE):
		"""Updates group information."""

		try:

			groups = REQUEST['groups']

		except KeyError:

			# No groups were selected

			groups = []

		if not type(groups) == type([]):

			groups = [groups]

		self.groups = groups

		RESPONSE.redirect(self.absolute_url() + '/')


	def __init__(self, id, first_name='', middle_name='', last_name='', street='', city='',
				state='', zip='', country='', email='', website='', telephone_home='',
				telephone_mobile='', telephone_pager='', telephone_fax='', note='',
				assistant_name='', assistant_phone='', assistant_email='',
				company_name='', company_street='', company_city='', company_state='', company_zip='',
				company_country='', company_employee_job_title='', company_employee_department='',
				company_employee_pager='', company_employee_email='', company_employee_phone='',
				company_employee_fax=''):
		"Initialize the class instance"

		self.id = id
		self.first_name = first_name
		self.middle_name = middle_name
		self.last_name = last_name
		self.street = street
		self.city = city
		self.state = state
		self.zip = zip
		self.country = country
		self.email = email
		self.website = website
		self.telephone_home = telephone_home
		self.telephone_mobile = telephone_mobile
		self.telephone_pager = telephone_pager
		self.telephone_fax = telephone_fax
		self.note = note
		self.assistant_name = assistant_name
		self.assistant_phone = assistant_phone
		self.assistant_email = assistant_email
		self.company_name = company_name
		self.company_street = company_street
		self.company_city = company_city
		self.company_state = company_state
		self.company_zip = company_zip
		self.company_country = company_country
		self.company_employee_job_title = company_employee_job_title
		self.company_employee_department = company_employee_department
		self.company_employee_pager = company_employee_pager
		self.company_employee_email = company_employee_email
		self.company_employee_phone = company_employee_phone
		self.company_employee_fax = company_employee_fax

		self.groups = []


	def render_website(self):
		"""Renders a browsable string if the website is set."""

		if not string.strip(self.website):

			return ''

		if string.find(self.website, 'http://') or \
			string.find(self.website, 'https://'):

			website_url = self.website

		else:

			website_url = 'http://' + self.website

		return """<a href="%s">%s</a>""" % (website_url, self.website)


	def copy(self, RESPONSE):
		"""Copies self to a new instance and redirects to it."""

		new_object = self.getParentNode().manage_clone(self, str(self.get_unique_id()))

		new_object.first_name = self.first_name + ' (copy)'
		new_object.middle_name = self.middle_name + ' (copy)'
		new_object.last_name = self.last_name + ' (copy)'

		RESPONSE.redirect(new_object.absolute_url() + '/base_edit')


default__class_init__(address_book_contact)
