###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.13 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS
import OFS.Folder
import Products
import base_item
import mail_gateway
import mailbox
import thread
import cStringIO
import re
import message_container
import news_group_folder
import message_filter_folder
import imap4_wrapper
import pop3_wrapper
import smtplib
import string

from Products import RFC822Message

gum_add_redirect = HTMLFile('gum_add_redirect', globals())
manage_add_gum_form = HTMLFile('gum_add', globals())


def manage_add_gum(self, id, local_username, email_address, title,
				local_password='local_password',
				mailhost='localhost',
				smtphost='localhost',
				REQUEST=None, RESPONSE=None):
    "Add an instance of the gum class"

    self._setObject(id, gum(id, local_username, email_address, mailhost,
				smtphost, local_password, title))

    if REQUEST is not None:

		RESPONSE.redirect(self.absolute_url() + '/' + id)


class gum(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	counter = 0

	message_container_types = message_container.message_container.types

	meta_type = 'GUM'
	icon = 'misc_/GUM/Folder_icon.gif'
	#default_catalog = 'Catalog'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'local_username', 'type':'string'},
		{'id':'local_password', 'type':'string'},
		{'id':'local_protocol', 'type':'string'},
		{'id':'mailhost', 'type':'string'},
		{'id':'smtphost', 'type':'string'},
		{'id':'email_address', 'type':'string'},
		{'id':'default_newshost', 'type':'string'},
		{'id':'listing_size', 'type':'int'},
		{'id':'signature', 'type':'text'},
		)

	meta_types = (
		{'name':'MailGateway',
		 'action':'manage_add_mail_gateway_form'},
		{'name':'MessageContainer',
		 'action':'manage_add_message_container_form'},
		{'name':'NewsGroupFolder',
		 'action':'manage_add_news_group_folder_form'},
		{'name':'MessageFilterFolder',
		 'action':'manage_add_message_filter_folder_form'}
		)

	manage_add_mail_gateway_form = mail_gateway.manage_add_mail_gateway_form
	manage_add_mail_gateway = mail_gateway.manage_add_mail_gateway
	mail_gateway_add_redirect = mail_gateway.add_mail_gateway_redirect

	manage_add_message_container_form = message_container.manage_add_message_container_form
	manage_add_message_container = message_container.manage_add_message_container
	message_container_add_redirect = message_container.message_container_add_redirect

	manage_add_news_group_folder_form = news_group_folder.manage_add_news_group_folder_form
	manage_add_news_group_folder = news_group_folder.manage_add_news_group_folder
	news_group_folder_add_redirect = news_group_folder.news_group_folder_add_redirect

	manage_add_message_filter_folder_form = message_filter_folder.manage_add_message_filter_folder_form
	manage_add_message_filter_folder = message_filter_folder.manage_add_message_filter_folder
	message_filter_folder_add_redirect = message_filter_folder.message_filter_folder_add_redirect	

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add MailGateway',
		 ('manage_add_mail_gateway', 'manage_add_mail_gateway_form')),
		('Add MessageContainer',
		 ('manage_add_message_container', 'manage_add_message_container_form')),
		('View GUMs',
		 ('',)),
		)

	index_html = HTMLFile('gum_index', globals())

	#base_edit = HTMLFile('gum_edit', globals())

	preferences = HTMLFile('gum_preferences', globals())

	display = HTMLFile('gum_display', globals())

	message_container_global_view = HTMLFile('gum_message_container_global_view', globals())
	message_filter_priorities = HTMLFile('gum_message_filter_priorities', globals())

	product_name = 'GUM'


	def __init__(self, id, local_username, email_address, mailhost, smtphost,
				local_password,
				local_protocol, title='',
				primary_folder='Incoming',
				signature=''):
				
		"Initialize the class instance"

		# Add Inbox and Outgoing

		message_container.manage_add_message_container(self, 'Incoming', 'Incoming messages', type=[0])
		message_container.manage_add_message_container(self, 'Outgoing',
								'Outgoing messages', type=[1])		
		message_container.manage_add_message_container(self, 'Drafts', 'Templates that save time', type=[2])
		message_container.manage_add_message_container(self, 'Temporary',
								'Temporary', type=[3])

		#message_filter_folder.manage_add_message_filter_folder(
		#	self, 'MessageFilters', 'MessageFilters')

		#mail_gateway.manage_add_mail_gateway(self, 'localhost', 'pop3', ''
		#					'localhost', '', 'morten-3', 'morten-2', 'morten-1')

		"""
		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_header',
					'standard_html_header',
					 open('%s/Products/%s/standard_html_header.dtml' % \
						(SOFTWARE_HOME, self.product_name)).read())
										  
		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_footer',
					'standard_html_footer',
					open('%s/Products/%s/standard_html_footer.dtml' % \
						(SOFTWARE_HOME, self.product_name)).read())

		OFS.DTMLMethod.addDTMLMethod(self, 'standard_css', 'standard_css',
					open('%s/Products/%s/standard_css.dtml' % \
						(SOFTWARE_HOME, self.product_name)).read())
		"""

		self.id = id
		self.title = title

		self.local_username = local_username
		self.local_password = local_password
		self.local_protocol = local_protocol
		self.mailhost = mailhost
		self.smtphost = smtphost
		self.email_address = email_address
		self.signature = signature
		self.default_newshost = ''

		self.primary_folder = primary_folder

		self.listing_size = 15

		try:
		
			self.index_object()

		except AttributeError:

			# No ZCatalog instance available.

			pass

		self.priority_count = 0


	def get_new_priority(self):
		"""Returns a new priority."""

		self.priority_count = self.priority_count + 1

		return self.priority_count


	def render_message_container_type(self, message_container_type):
		"""Renders the name of the message container type."""

		return self.message_container_types[int(message_container_type)]


	def render_message_container_title(self, message_container_id):
		"""Renders the name of the message container."""

		return self[message_container_id].title


	def get_unique_id(self):
		"""Returns a unique identifier."""

		try:

			return self.getParentNode().get_unique_id()

		except AttributeError:

			self.counter = self.counter + 1

			return str(self.counter)


	def get_id(self):
		"""Hack, do not use this method."""

		# FIXME

		print 'get_id was used..'
		return self.get_unique_id()


	def get_primary_folder(self):
		"""Returns the primary folder."""

		# FIXME
		# We need to add a primary folder selection..

		return self.get_message_containers(type=0)[0]

		#return self[self.primary_folder]


	def get_temporary_folder(self):
		"""Returns the temporary folder."""

		return self.get_message_containers(type=3)[0]


	def get_draft_folder(self):
		"""Returns the draft folder."""

		# FIXME

		return self['Drafts']


	def get_gum(self):
		"""Returns the GUM instance."""

		return self


	def get_mail_gateway(self):
		"""Returns default mail gateway."""

		try:

			return self[self.default_mail_gateway]

		except AttributeError:

			return self.objectValues('MailGateway')[0]


	def get_news_host(self):
		"""Returns newshost."""

		# FIXME -- this is not (does not feel like) the right
		# way to do it..

		news_hosts = []

		reference = self

		while 1:

			news_hosts = reference.objectValues('NewsHost')

			if news_hosts:

				break

			reference = reference.getParentNode()

		if not news_hosts:

			raise 'NoNewsHosts', 'No news hosts are configured on this server'

		return news_hosts[0]


	def get_gum_version(self):
		"""Returns a version number."""

		return open(SOFTWARE_HOME + '/Products/' + self.product_name \
					+ '/version.txt').read()


	def get_gum_website(self):
		"""Returns the GUM website."""

		return open(SOFTWARE_HOME + '/Products/' + self.product_name \
				+ '/website.txt').read()


	def get_address_book(self):
		"""Returns an AddressBook instance if available."""

		object =  self.get_nearest_object_by_meta_type('AddressBook')

		return object


	def get_number_of_mail_messages(self):
		"""Gets the number of unread messages in all incoming mailboxes."""

		count = 0

		for message_container in self.get_message_containers(type=0):

			count = count + message_container.get_number_of_messages()

		return count


	def get_number_of_unread_mail_messages(self):
		"""Gets the number of unread messages in all mailboxes."""

		count = 0

		for message_container in self.objectValues('MessageContainer'):

			count = count + message_container.get_number_of_unread_messages()

		return count


	def get_message_filters(self):
		"""Returns all message filters sorted by priority."""

		message_filters = []

		for message_container in self.get_message_containers():

			message_filters.extend(message_container.get_message_filters())

		message_filters.sort(lambda x,y: cmp(x.priority, y.priority))

		return message_filters


	def render_message_container_type_widget(self, REQUEST):
		"""Renders an HTML widget."""

		select_widget = """<select name="message_container_type:int">"""

		for option in self.message_container_types.keys():

			if REQUEST['message_container_type'] == option:

				selected = " selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s"%s>%s</option>""" % \
						(option, selected, self.message_container_types[option])

		return select_widget + "</select>"


	def list_message_containers(self):
		"""Returns list of message containers."""

		message_containers = []

		message_container_types = self.message_container_types.keys()
		message_container_types.sort()

		for container_type in message_container_types:

			message_containers.append({'group_title':self.message_container_types[container_type],
							'group_global_view_url':self.absolute_url() + \
								'/message_container_global_view?' + \
								'message_container_type=%s' % container_type,
							'containers':self.get_message_containers(type=container_type)})

		return message_containers


	def retrieve_messages(self, REQUEST=None, RESPONSE=None,
				run_as_thread=0):
		"""Wrapper so we can choose whether to retrieve messages
		in threaded mode."""

		if run_as_thread:

			thread.start_new_thread(self._retrieve_messages,
				(self,),
				{'REQUEST':REQUEST, 'RESPONSE':RESPONSE})

		else:

			self._retrieve_messages(self, REQUEST=REQUEST,
							RESPONSE=RESPONSE)

		RESPONSE.redirect(self.absolute_url() + '/')
	

	def _retrieve_messages(self, hack, REQUEST=None, RESPONSE=None):
		"""Retrieves messages from the local server."""

		message_filters = self.get_message_filters()

		primary_folder = self.get_primary_folder()

		#mbox = mailbox.UnixMailbox(open('%s/Products/%s/test-mailbox' % \
		#				(SOFTWARE_HOME, self.product_name)))

		# FIXME -- poor hack
		# Move to updating of properties?

		if self.local_username and self.local_password and self.local_protocol:

			if self.local_protocol == 'imap':

				con = imap4_wrapper.imap4_wrapper(self.mailhost)

			elif self.local_protocol == 'pop3':

				con = pop3_wrapper.pop3_wrapper(self.mailhost)

			else:

				raise 'UnknownProtocolException', self.local_protocol

			con.login(self.local_username, self.local_password)

			mbox = con.get_all_messages(RESPONSE=RESPONSE)

		else:

			checkname = re.compile(r'^([A-Z]|[a-z]|[0-9])*')

			if not checkname.search(self.local_username):

				raise 'NoWay', 'You\'re not getting away with this username.. %s'\
						% self.local_username

			mbox = mailbox.UnixMailbox(
				open('/var/spool/mail/'+self.local_username))

			#mbox = mailbox.UnixMailbox(
			#	open('/tmp/mailbox'))

		message_count = 1

		while 1:

			if not message_count % 5:

				self.really_commit_transaction()

				if RESPONSE is not None:

					RESPONSE.write('5')

				if not message_count % 100:

					RESPONSE.write('\n')

			_message = mbox.next()

			print _message

			if not _message:

				break

			_message.fp.seek(0)
			_message = RFC822Message.util.my_string_io(_message.fp.read())

			message = RFC822Message.rfc822_message_parse.rfc822_message_parse(
				_message)

			for message_filter in message_filters:

				if message_filter.match(message.headers):

					RFC822Message.util.store_message(
						message_filter.getParentNode(),
						message, readonly=1)

					break

			else:

				RFC822Message.util.store_message(
					primary_folder, message, readonly=1)

			message_count = message_count + 1


	def gum_enviroment(self):
		"""Returns a truth value to whether the requesting object is within a
		GUM enviroment."""

		# FIXME -- this is stupid

		return 1


	def compose_mail(self, REQUEST=None, RESPONSE=None):
		"""Creates a new instance of a message and redirects to it."""

		subject = ''
		body = ''
		recipients = []
		newsgroups = []

		if REQUEST:

			if string.strip(self.signature):

				signature_append = '\n--\n' + self.signature

			else:

				signature_append = ''

			if REQUEST.has_key('subject') and REQUEST['subject']:

				subject = subject

			if REQUEST.has_key('body') and REQUEST['body']:

				body = body + signature_append

			else:

				body = body + signature_append

			if REQUEST.has_key('recipients') and REQUEST['recipients']:

				recipients = REQUEST['recipients']

				if type(recipients) == type(''):

					recipients = [recipients]

			if REQUEST.has_key('newsgroups') and REQUEST['newsgroups']:

				newsgroups = REQUEST['newsgroups']

				if type(newsgroups) == type(''):

					newsgroups = [newsgroups]


		temporary_folder = self.get_temporary_folder()

		new_id = str(self.get_unique_id())

		RFC822Message.rfc822_message.manage_add_rfc822_message(temporary_folder,
								new_id,
								sender=self.email_address,
								body=body,
								subject=subject)

		message_reference = temporary_folder[new_id]

		if recipients:

			for recipient in recipients:

				new_id = str(self.get_unique_id())

				message_reference['to'].manage_add_rfc822_address(
									new_id,
									recipient)

		if newsgroups:

			for newsgroup in newsgroups:

				message_reference.add_newsgroup(newsgroup)


		if RESPONSE:

			RESPONSE.redirect(message_reference.absolute_url())

		else:

			return message_reference


	def get_mailboxes(self):
		"""Wrapper, don't use this -- FIXME."""

		return self.get_message_containers()


	def get_message_containers(self, type=None):
		"""Returns a list of all available message containers."""

		message_containers = self.objectValues('MessageContainer')

		if type is not None:

			filter_function = "lambda x: x.type == %s" % type
	
			truth_filter = eval(filter_function)
	
			message_containers = filter(truth_filter, message_containers)

		return message_containers


	def manage_preferences(self, REQUEST, RESPONSE):
		"""Manages preferences."""

		self.base_edit_properties(REQUEST)


	def get_all_messages_in_container_type(self, type):
		"""Returns all messages in given container type."""

		message_containers = []
		messages = []

		for message_container in self.get_message_containers(type=type):

			for message in message_container.get_messages():

				messages.append(message)

		return messages


	def get_all_available_newshosts(self):
		"""Returns a list of all available news_host objects."""

		news_hosts = []

		reference = self

		while 1:

			try:

				_news_hosts = reference.objectValues('NewsHost')

			except AttributeError:

				# We've come to the end of the road..

				break

			if _news_hosts:

				news_hosts.extend(_news_hosts)

			reference = reference.getParentNode()

		return news_hosts


	def move_message_filter_up(self, REQUEST, RESPONSE):
		"""Increases the priority of given message filter."""

		priority = int(REQUEST['priority'])
		message_filters = self.get_message_filters()

		for message_filter_index in range(len(message_filters)):

			message_filter = message_filters[message_filter_index]

			if message_filter.priority == priority:

				downgraded_filter = message_filters[message_filter_index - 1]

				new_priority = downgraded_filter.priority
				message_filter.priority = new_priority
				downgraded_filter.priority = priority

				break

		RESPONSE.redirect(self.absolute_url() + '/message_filter_priorities')


	def move_message_filter_down(self, REQUEST, RESPONSE):
		"""Decreases the priority of given message filter."""

		priority = int(REQUEST['priority'])
		message_filters = self.get_message_filters()

		for message_filter_index in range(len(message_filters)):

			message_filter = message_filters[message_filter_index]

			if message_filter.priority == priority:

				try:

					upgraded_filter = message_filters[message_filter_index + 1]

				except IndexError:

					upgraded_filter = message_filters[0]

				new_priority = upgraded_filter.priority
				message_filter.priority = new_priority
				upgraded_filter.priority = priority

				break


		RESPONSE.redirect(self.absolute_url() + '/message_filter_priorities')


	def build_smtp(self):
		"""Reopens any of the connections if necessary."""

		return smtplib.SMTP(self.smtphost)


	def post(self, message, raw_message=None):
		"""Deconstructs an rfc822_message object and posts it."""

		smtp_connection = self.build_smtp()

		if raw_message is None:

			raw_message = message.raw_message()

		smtp_connection.sendmail(message.sender, message.get_recipients(),
						raw_message)

		smtp_connection.quit()


default__class_init__(gum)