###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.4 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import MessageDialog
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import rfc822_address
import base_item


rfc822_address_container_add_redirect = HTMLFile('rfc822_address_container_add_redirect', globals())
manage_add_rfc822_address_container_form = HTMLFile('rfc822_address_container_add', globals())


def manage_add_rfc822_address_container(self, id, title, REQUEST=None):
    "Add an instance of the rfc822_address_container class"

    self._setObject(id, rfc822_address_container(id, title))

    if REQUEST is not None:

        return self.manage_main(self, REQUEST)


class rfc822_address_container(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'RFC822AddressContainer'
	icon = 'misc_/RFC822Message/dtmldoc.gif'

	_properties = (
		{'id':'title', 'type':'string'},
		)

	meta_types = (
		{'name': 'RFC822Address', 'action':'manage_add_rfc822_address_form'},
		)

	manage_add_rfc822_address_form = rfc822_address.manage_add_rfc822_address_form
	manage_add_rfc822_address = rfc822_address.manage_add_rfc822_address
	rfc822_address_add_redirect = rfc822_address.rfc822_address_add_redirect
	
	r = rfc822_address_address_book_interface = rfc822_address.rfc822_address_address_book_interface

	manage_options = (
		{'label':'Edit', 'action':'manage_main'},
		{'label':'View', 'action':''},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Attachment',
		 ('','render_data')),
		)

	index_html = HTMLFile('rfc822_address_container_index', globals())
	#manage_main = HTMLFile('rfc822_address_container_edit', globals())


	def __init__(self, id, title=''):
		"Initialize the class instance"

		self.id = id

		self.title = title

		self.index_object()


	def add_normal_email(self, REQUEST, RESPONSE):
		"""Adds all normal emails specified."""

		if REQUEST.has_key('ids'):

			for id in REQUEST['ids']:

				self.manage_add_rfc822_address(str(self.get_unique_id()),
							self.get_address_book()[id].email)

		# Redirect the the mail message.

		RESPONSE.redirect(self.getParentNode().absolute_url())
		

		#RESPONSE.redirect(self.absolute_url() + '/rfc822_address_address_book_interface')


	def add_company_email(self, REQUEST, RESPONSE):
		"""Adds all normal emails specified."""

		if REQUEST.has_key('ids'):

			for id in REQUEST['ids']:

				self.manage_add_rfc822_address(str(self.get_unique_id()),
							self.get_address_book()[id].company_employee_email)

		# Redirect the the mail message.

		RESPONSE.redirect(self.getParentNode().absolute_url())

		#RESPONSE.redirect(self.absolute_url() + '/rfc822_address_address_book_interface')


	def add(self, REQUEST, RESPONSE):
		"""Method with a short name for 'efficient use' in large HTML listings."""

		email = REQUEST['email']

		if REQUEST.has_key('realname'):

			realname = REQUEST['realname']

		else:

			realname = ''

		if REQUEST.has_key('comment'):

			comment = REQUEST['comment']

		else:

			comment = ''

		new_id = str(self.get_unique_id())

		self.manage_add_rfc822_address(new_id, email, realname=realname, comment=comment)

		RESPONSE.redirect(self.getParentNode().absolute_url())


	def add_address_and_update(self, REQUEST, RESPONSE):
		"""Updates the message and returns a add contact field."""

		self.getParentNode().update(REQUEST=REQUEST)

		RESPONSE.redirect(self.absolute_url() + '/rfc822_address_address_book_interface')


	def delete(self, REQUEST=None, RESPONSE=None):
		"""A wrapper for manage_delObjects so we can redirect back to
		the message."""

		try:

			ids = REQUEST['ids']

		except KeyError:

			# The user didn't specify any emails.

			RESPONSE.redirect(self.getParentNode().absolute_url())
			return

		self.manage_delObjects(ids=ids)

		if RESPONSE is not None:

			RESPONSE.redirect(self.getParentNode().absolute_url())


	def manage_edit(self, title, REQUEST=None):
		"Change properties for the class instance."

		self.title = title

		self.reindex_object()

		if REQUEST is not None:
			return MessageDialog(
			   title='Edited',
			   message="Properties for %s has been changed." % self.id,
			   action = "./manage_main"
			   )



default__class_init__(rfc822_address_container)

