###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import Products
import base_item
import util

manage_add_workspace_form = HTMLFile('workspace_add', globals())


def manage_add_workspace(self, id, title, description, REQUEST=None):
	"Add an instance of the bookmark_folder class"

	self._setObject(id, workspace(id, title, description))

	if REQUEST is not None:

		REQUEST.RESPONSE.redirect(self.absolute_url())


class workspace(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'Workspace'
	icon = 'misc_/Workspace/Folder_icon.gif'
	default_catalog = 'Catalog'
	product_name = 'Workspace'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Workspaces',
		 ('', 'get_globals_bokmark_folder',
			'get_gum', 'get_address_book',
			'get_address_book', 'get_note_folder',
			'get_note_folder', 'get_bookmark_folder',
			'get_calendar', 'get_personal_folder',
			'get_workspace_actions'),
			('Owner', 'Manager')),
		)


	index_html = HTMLFile('workspace_index', globals())
	#display = HTMLFile('workspace_display', globals())



	def get_global_bookmark_folder(self):
		"""HACK -- FIXME"""

		return self.get_bookmark_folder()


	def get_gum(self):
		"""Returns the mail client."""

		return self.objectValues('GUM')[0]


	def get_address_book(self):
		"""Returns the address book."""

		return self.objectValues('AddressBook')[0]


	def get_note_folder(self):
		"""Returns the notes."""

		return self.objectValues('Notes')[0]


	def get_bookmark_folder(self):
		"""Returns the bookmark folder."""

		return self.objectValues('BookmarkFolder')[0]

	def get_calendar(self):
		"""Returns the calendar."""

		return self.objectValues('CalendarFolder')[0]


	def get_personal_folder(self):
		"""Returns the personal folder."""

		return self.objectValues('PersonalFolder')[0]


	def get_todo_folder(self):
		"""Returns the todo folder."""

		return self.objectValues('TodoFolder')[0]


	def get_workspace_actions(self):
		"""Returns a list of 'jump-actions'."""

		action_marker = '- - - - '

		actions = [
				{'action':self.absolute_url(),
					'title':'Select action'}
			]
				

		gum = self.get_gum()

		actions.append(
			{
			'action':gum.relative_url(),
			'title':'Go to messenger'
			})

		actions.append(
			{
			'action':gum.relative_url() + '/compose_mail',
			'title':action_marker + 'Compose message'
			})

		actions.append(
			{
			'action':gum.get_primary_folder().relative_url(),
			'title':action_marker + gum.get_primary_folder().title
			})

		calendar = self.get_calendar()

		actions.append(
			{
			'action':calendar.get_today().relative_url(),
			'title':'Go to Calendar'
			})

		address_book = self.get_address_book()

		actions.append(
			{
			'action':address_book.relative_url(),
			'title':'Go to address book'
			})

		actions.append(
			{
			'action':address_book.relative_url() + \
				'/manage_add_address_book_contact_form',
			'title':action_marker + 'Add contact'
			})

		notes = self.get_note_folder()

		actions.append(
			{
			'action':notes.relative_url(),
			'title':'Go to notes'
			})

		actions.append(
			{
			'action':notes.relative_url() + '/note_add_without_context',
			'title':action_marker + 'Add note'
			})

		bookmarks = self.get_bookmark_folder()

		actions.append(
			{
			'action':bookmarks.relative_url(),
			'title':'Go to bookmarks'
			})

		return actions


	def get_unique_id(self):
		"""Returns a unique identifier."""

		try:

			return str(self.getParentNode().get_unique_id())

		except AttributeError:

			self.counter = self.counter + 1

			return str(self.counter)


	def __init__(self, id, title='', description=''):
		"Initialize the class instance"

		self.id = id
		self.title = title
		self.description = description

		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_header',
					'standard_html_header',
					 open('%s/Products/%s/standard_html_header.dtml' % \
						(SOFTWARE_HOME, self.product_name)).read())
										  
		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_footer',
					'standard_html_footer',
					open('%s/Products/%s/standard_html_footer.dtml' % \
						(SOFTWARE_HOME, self.product_name)).read())

		OFS.DTMLMethod.addDTMLMethod(self, 'standard_css', 'standard_css',
					open('%s/Products/%s/standard_css.dtml' % \
						(SOFTWARE_HOME, self.product_name)).read())


		util.install_workspace(self, username='morten', email='morten@esol.no')

		self.index_object()

		self.counter = 0


default__class_init__(workspace)

